//---------------------------------------------------------------------------
// jrdebug_blockhelpers.h
// functions that support hierarchical blocking of activities
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// debug block methods
JrDebugBlockObjectData::JrDebugBlockObjectData(const jrd_stdstringT str,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3, bool inisScopedelete)
	{
	// save params
	filenamestring=jrd_stdstringT(infilename);
	linenumber=ilinenum;
	timestampstring=jrd_stdstringT(intimestamp);
	// save start time
	startclock=clock();
	// is it pushed?
	isScopedelete=inisScopedelete;
	announceatend=false;
	// for reporting
	childerrors_block=0;
	childerrors_immediate=0;
	childerrors_all=0;
	childs=0;
	parentp=NULL;
	// label
	if (str!="")
		label=str;
	else
		label=functionnamestring;

	// function name, get the first one that matched
	if (infunctionname[0]!='\0')
		functionnamestring = jrd_stdstringT(infunctionname);
	else if (infunctionname2[0]!='\0')
		functionnamestring = jrd_stdstringT(infunctionname2);
	else
		functionnamestring = jrd_stdstringT(infunctionname3);

	// push it on static list (this will set parentp) as long as isScopedelete==true
	if (isScopedelete)
		{
		// announce it?
		// ATTN: we want to only do this if label dont start with '*' and isScopedelete==true
		report_start();
		// add it to our stack
		JrDebug::DebugBlockPushPointer(this);
		}
	}

JrDebugBlockObjectData::~JrDebugBlockObjectData()
	{
	// object dies; adjust parent count
	if (parentp!=NULL)
		parentp->AddErrors(childerrors_immediate,childerrors_all,childerrors_block);
	// pop it from static list, IF it is an autoscope creation
	if (this->get_isScopedelete())
		JrDebug::DebugBlockPopPointer(this);
	}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
void JrDebugBlockObjectData::AddErrors(int inchilderrors_immediate, int inchilderrors_all, int inchilderrors_block)
{
	// add erros from children
	childerrors_all+=inchilderrors_all;
	// if child had any of its own errors, than we increase immediate count
	if (inchilderrors_immediate>0 || inchilderrors_block)
		++childerrors_block;

	// keep track of number of childs
	++childs;
}

void JrDebugBlockObjectData::report(const char* labelstr)
	{
	// report (used to do a quick announcement of time in current block)
	clock_t endclock = clock();
	float elapsed = (float)(endclock - startclock)/(float)CLOCKS_PER_SEC;
	JrDebug(filenamestring.c_str(),linenumber,timestampstring.c_str(),functionnamestring.c_str(),"","",JrdMessage)(JrdBlockInfo,"","",JrDebugDefaultLevel,"At %s:%s (%f seconds)",label.c_str(),labelstr,elapsed);
	}

void JrDebugBlockObjectData::report_start()
	{
	JrDebug(filenamestring.c_str(),linenumber,timestampstring.c_str(),functionnamestring.c_str(),"","",JrdMessage)(JrdBlockStart,"","",JrDebugDefaultLevel,"Started %s",label.c_str());
	announceatend=true;
	}

void JrDebugBlockObjectData::report_end()
	{
	// announce its death if it was announced during push
	JrdType mtype;
	clock_t endclock = clock();
	float elapsed = (float)(endclock - startclock)/(float)CLOCKS_PER_SEC;
	// results
	jrd_stdstringT resultstring;
	char resultnumbuf[255];
	if (childs>0 && childerrors_all>0)
		{
		sprintf(resultnumbuf," (%.02f sec , %d err , %d/%d)",elapsed,childerrors_all,childerrors_block,childs);
		}
	else
		{
		if (childerrors_all==0)
			sprintf(resultnumbuf," (%.02f sec)",elapsed);
		else
			sprintf(resultnumbuf," (%.02f sec , %d err)",elapsed,childerrors_all);
		}
	//
	resultstring=label+" "+jrd_stdstringT(resultnumbuf);
	//
	if (childerrors_all==0)
		mtype=JrdBlockEnd;
	else
		mtype=JrdBlockEndError;
	//
	JrDebug(filenamestring.c_str(),linenumber,timestampstring.c_str(),functionnamestring.c_str(),"","",JrdMessage)(mtype,"","",JrDebugDefaultLevel,"Finished %s.",resultstring.c_str());
	// now mark it as ended
	announceatend=false;
	}

void JrDebugBlockObjectData::report_error(const char *str)
	{
	// report an error for a debug block
	JrDebug(filenamestring.c_str(),linenumber,timestampstring.c_str(),functionnamestring.c_str(),"","",JrdMessage)(JrdBlockError,"","",0,"%s (block=%s)",str,label.c_str());
	}


bool JrDebugBlockObjectData::doesMatchLabel(const char* labelstr)
	{
	if (strcmp(labelstr,label.c_str())==0)
		return true;
	// ATTN: do we want to check for '*' stuff too?
	char *labelstr2 = (char*)labelstr;
	if (labelstr2[0]=='*')
		{
		if (strcmp(&labelstr2[1],label.c_str())==0)
			return true;
		}
	// no match
	return false;
	}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// block report helpers
bool JrDebug::DebugBlockPushPointer(JrDebugBlockObjectData *datum)
	{
	// push the pointer onto our static queue

	// check to make sure we are not too deep
	//  (if we are then we should throw an error of some sort)
	if (debugblockdepth>=JrDebug_MaxDebugBlockDepth)
		{
		// ATTN: we want to use filename from datum
		datum->report_error("Too many nested Activity/Debug Blocks, ie failure to match pops+pushes");
		return false;
		}
	// set parent pointer and make this the last in the stack
	datum->set_parentp(lastblockobjp);
	lastblockobjp=datum;
	++debugblockdepth;
	return true;
	}

bool JrDebug::DebugBlockPopPointer(JrDebugBlockObjectData *datum)
	{
	// pop the pointer (the last stack pointer SHOULD be == datum (note we dont delete datum)
	// if its not, it means things got out of sync because user fucked up, so we unwind until we hit datum, and report the desync
	if (lastblockobjp!=datum)
		{
		// this situation is an error, just a matter of how bad
		// we will try to walk back in stack and find it (but dont delete anything until we are sure)
		JrDebugBlockObjectData *curp=lastblockobjp;
		while (curp!=NULL)
			{
			curp=curp->get_parentp();
			if (curp==datum)
				break;
			}
		if (curp==NULL)
			{
			// it wasnt found at all, complain more but do nothing
			datum->report_error("Tried to pop a nested Activity/Debug Block which was not found");
			return false;
			}
		// error (warning)
		datum->report_error("Popping a nested Activity/Debug Block, which was not at top of stack");

		// ok we found it, so now walk back again and remove intervening ones
		curp=lastblockobjp;
		JrDebugBlockObjectData *parp;
		while (curp!=NULL)
			{
			parp=curp->get_parentp();
			--debugblockdepth;
			// announce end
			if (curp->get_announceatend())
				curp->report_end();
			// free it without fanfare or announcement, IFF its not a scoped object which will kill itself when it goes out of scope
			if (!(curp->get_isScopedelete()))
				delete curp;
			// now loop to next
			curp=parp;
			if (curp==datum)
				break;
			}
		// ok so now the top of stack is the current one we want to close
		lastblockobjp=curp;
		}
	// ok if we are here, then our datum is at top of stack ready to be popped (and announced)
	// pop it off
	lastblockobjp=datum->get_parentp();
	--debugblockdepth;
	// announced its completion if appropriate
	if (datum->get_announceatend())
		datum->report_end();
	// NOTE: we dont delete this datum - caller has to!
	return true;
	}


JrDebugBlockObjectData* JrDebug::DebugBlockPopPointer(const char *labelstr,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3)
	{
	// pop the block, based on label
	// this just proxies to the pointer version
	JrDebugBlockObjectData *curp=lastblockobjp;
	while (curp!=NULL)
		{
		// if we found it then this is it
		if (curp->doesMatchLabel(labelstr))
			break;
		// if we hit a scoped object, then we dont travel above it, since we know this is an error
		if (curp->get_isScopedelete())
			{
			curp=NULL;
			break;
			}
		// travel up
		curp=curp->get_parentp();
		}
	if (curp==NULL)
		{
		// error not found
		JrDebug(infilename,ilinenum,intimestamp,infunctionname,infunctionname2,infunctionname3,JrdMessage)(JrdBlockError,"","",0,"Failed to pop a nested Activity/Debug Block (%s), which was not found.",labelstr);
		return NULL;
		}
	// found (this shouldnt return false but we check just in case)
	if (DebugBlockPopPointer(curp))
		return curp;
	// error not deleted
	JrDebug(infilename,ilinenum,intimestamp,infunctionname,infunctionname2,infunctionname3,JrdMessage)(JrdBlockError,"","",0,"Failed to pop Activity/Debug Block (%s), which was not found.",labelstr);
	return NULL;
	}


void JrDebug::DebugBlockReport(const char *labelstr)
	{
	// report using latest debug block
	if (lastblockobjp!=NULL)
		lastblockobjp->report(labelstr);
	}
//---------------------------------------------------------------------------











//---------------------------------------------------------------------------
// activity setting
void JrDebug::SetActivity(const char *inactivity2,const JrdType inmtype,JrDebugRegisteredTestObj *rtop,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3)
	{
	// set an activity
	// touch unused args to avoid compiler warnings
	JrdUnusedArg(inactivity2); JrdUnusedArg(inmtype); JrdUnusedArg(rtop); JrdUnusedArg(infilename); JrdUnusedArg(ilinenum); JrdUnusedArg(intimestamp); JrdUnusedArg(infunctionname); JrdUnusedArg(infunctionname2);JrdUnusedArg(infunctionname3);
	}


void JrDebug::PushActivity(const char *inactivity2,const JrdType inmtype,JrDebugRegisteredTestObj *rtop,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3)
	{
	// push an activity
	// the new version should dynamically create a debug block object and push it on stack
	JrDebugBlockObjectData* datum;
	// create and note the its not autoscope deleted
	if (rtop==NULL)
		datum= new JrDebugBlockObjectData(inactivity2, infilename , ilinenum , intimestamp ,  infunctionname, infunctionname2,infunctionname3,false);
	else
		datum= new JrDebugBlockObjectData(inactivity2,(rtop->getFilename()).c_str() , rtop->getLinenum() , intimestamp , (rtop->getFname()).c_str(), "", "",false);

	// note: when created it will it push itself?  no because we are passing isScopedelete as false
	// so we push it here and announce it
	datum->report_start();
	DebugBlockPushPointer(datum);
	// touch unused args to avoid compiler warnings
	JrdUnusedArg(inmtype);
	}


void JrDebug::PopActivity(const char *inactivity2,const JrdType inmtype,JrDebugRegisteredTestObj *rtop,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3)
	{
	// pop off the last activity
	// the new version should dynamically pop the object off stack, AND DELETE IT (since it doesnt have its own scope deletion)
	JrDebugBlockObjectData* datum;
	datum=DebugBlockPopPointer(inactivity2,infilename,ilinenum , intimestamp ,  infunctionname, infunctionname2, infunctionname3);
	if (datum)
		{
		// ok it was popped - now we must delete (since its not an auto scoped object)
		delete datum;
		}
	else
		{
		// error but user will have been announced in poppointer above
		}
	// touch unused args to avoid compiler warnings
	JrdUnusedArg(inmtype);
	JrdUnusedArg(rtop);
	}


// set activity mod
void JrDebug::SetActivityModString(const char *inactivity)
	{
	// for setting activity of a current command
	activitymodstring=inactivity;
	}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
void JrDebug::PopAllDebugBlocks()
	{
	// pop all blocks on close
	JrDebugBlockObjectData *curp=lastblockobjp;
	JrDebugBlockObjectData *parp;
	while (curp!=NULL)
		{
		parp=curp->get_parentp();
		// free it without fanfare or announcement, IFF its not a scoped object which will kill itself when it goes out of scope
		if (curp->get_announceatend())
			curp->report_end();
		if (!(curp->get_isScopedelete()))
			delete curp;
		--debugblockdepth;
		// now loop to next
		curp=parp;
		}
	// ok so now the top of stack is the current one we want to close
	lastblockobjp=curp;
	}
//---------------------------------------------------------------------------
