//---------------------------------------------------------------------------
// jrdebug_classes.h
// class definitions
// ATTN: I think we should be able to make a replacement minimal copy of these
//       for the case where compiling is not compiled in.
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// Define Constants
// default level used for messages when none is specified
#define JrDebugDefaultLevel 50
// magic# passed as an arg to ignore
#define JrDebugIgnoreLevel  -98745
// max debug depth before we report likely error
#define JrDebug_MaxDebugBlockDepth 500
// max commandline args
#define JrDebug_MaxCommandlineLargs 255
// max buffer size used for max debug string length,etc
#define JrDebug_MaxBufferSize 4096
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// forward declarations
class JrDebugRegisteredTestObj;
class JrDebugRegisteredOutputFormatter;
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// main debug logging class
class JrDebug
{
private:
	// created each time on normal construct
	JrdType messagetype;
	int depth;
	// 
	jrd_stdstringT filenamestring;
	jrd_stdstringT filedirstring;
	jrd_stdstringT functionnamestring;
	jrd_stdstringT timestampstring;
	jrd_stdstringT messagetypestring;
	jrd_stdstringT activitymodstring;
	int severitylevelnumber;
	int linenumber;
	bool shouldshowmessagebox;
	bool singletonflag;
protected:
	// statics
	static long debugindex;
	static jrd_stdstringT currentactivitystring;
	static long runid;
	static jrd_stdstringT defaultoutputoptions;
	static jrd_stdstringT programdescription;
protected:
	// statics for win32 console special handling
	static bool did_checkwin32console;
	static bool did_allocwin32console;
protected:
	// activity/debug blocks
	static class JrDebugCallback *jrcallbackp;
	static class JrDebugBlockObjectData* lastblockobjp;
	static int debugblockdepth;
	// testing objects
	static JrDebugRegisteredTestObj* testobjects;
	static JrDebugRegisteredTestObj* lasttestobjp;
	static JrDebugRegisteredTestObj* curtestobjp;
	static bool rantests;
	// output formatters
	static JrDebugRegisteredOutputFormatter* formatter_factorys;
	static JrDebugRegisteredOutputFormatter* lastformatter_factoryp;
	static JrDebugRegisteredOutputFormatter* formatters;
	static JrDebugRegisteredOutputFormatter* lastformatterp;
public:
	// we make these public so we can access directly and to eliminate all possibility of the compiler not inlining access to it
	static bool enabled;
	static int exceptionlevel;
protected:
	// filters
	static bool displaypassingasserts;
	static int alarmfilterlevel;
	static int filtermin;
	//
	static bool filter_showallblocks;
	static bool filter_onlyerrors;
public:
	// constructor (for global singleton) and destructor
	JrDebug(bool doinit);
	~JrDebug();
	// constructor called by our clever macros
	JrDebug(const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3, JrdType inmtype);
public:
	static void RemoveInstantiatedFormatters();
public:
	// activity
	static void SetActivity(const char *inactivity2,const JrdType inmtype,JrDebugRegisteredTestObj *rtop,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3);
	static void PushActivity(const char *inactivity2,const JrdType inmtype,JrDebugRegisteredTestObj *rtop,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3);
	static void PopActivity(const char *inactivity2,const JrdType inmtype,JrDebugRegisteredTestObj *rtop,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3);
public:
	// accessors
	static void SetFilterLevel(int minval);
	static bool GetEnabled();
	static void SetEnabled(bool val);
	static void SetEnabled(bool val,char *reasonstr,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3);
	static void SetDisplayPassingAsserts(bool val);
	static void SetAlarmFilterLevel(int val);
	static void SetExceptionLevel(int val);
	static void DisableAlarmsFilter();
	static void DisableExceptionLogging();
	static void EnableExceptionLogging(int val=JrDebugDefaultLevel);
	static void SetCallbackPointer(class JrDebugCallback *inp);
	static jrd_stdstringT get_programdescription() {return programdescription;};
	static void set_defaultoutputoptions(jrd_stdstringT inval) {defaultoutputoptions=inval;};
public:
	// accessors used by output formatters
	jrd_stdstringT get_currentactivitystring() {return currentactivitystring;};
	jrd_stdstringT get_activitymodstring() {return activitymodstring;};
	jrd_stdstringT get_filenamestring() {return filenamestring;};
	jrd_stdstringT get_filedirstring() {return filedirstring;};
	jrd_stdstringT get_functionnamestring() {return functionnamestring;};
	jrd_stdstringT get_messagetypestring() {return messagetypestring;};
	jrd_stdstringT get_timestampstring() {return timestampstring;};
	long get_linenumber() {return linenumber;};
	long get_runid() {return runid;};
	int get_severitylevelnumber() {return severitylevelnumber;};
	long get_debugindex() {return debugindex;};
	JrdType get_messagetype() {return messagetype;};
	int get_depth() {return depth;};
	//
	bool MessageTypeIsBlock() {if (messagetype==JrdBlockStart || messagetype==JrdBlockEnd || messagetype==JrdBlockEndError) return true; return false; };
	bool MessageIsSimpleError();
	bool MessageIsError();
	bool FilterBlock();
public:
	// helper that returns true if there are any formatters active for stdout
	static bool isAnyStdoutEnabled();
public:
	// commandline parssing functions
	static bool ParseCommandline(char *inprogramdesc,int *argcp,char **argv, char* altcomlinestr, char *precomline, char* defaultcomline, char *indefaultoutputoptions);
	static bool ParseCommandlineArgString(const char *commandline,char *arg0,bool *foundcommandlineargsp);
	static void Announce(const char *mesg="",bool evenifdisabled=true,int argc=0,char **argv=NULL);
protected:
	// commandline parsing helpers
	static bool FindAndGrabCommandlineArg(int &argc,char **argv,const jrd_stdstringT &flagstring,jrd_stdstringT &extrastring, const jrd_stdstringT &defaultextraval, bool grabextra, bool requiresextra);
	static bool ParseCommandlineArgcArgv(int &argc,char **argv,bool *foundcommandlineargsp);
	static bool ParseOutputOption(jrd_stdstringT &dbostring, jrd_stdstringT &stylename, jrd_stdstringT &optionstring, jrd_stdstringT &filename,bool &stdoutflag, bool &systemoutflag, bool &debuginfoflag);
	static void CommandlineParseMessage(const jrd_stdstringT &messagestring);
	static bool SplitStringLeft(jrd_stdstringT &mainstring,jrd_stdstringT &leftpart,char dividerchar);
	static bool SetFilterOptions(jrd_stdstringT optionstring);
	//
	static jrd_stdstringT AddSmartPathToOutputFile(jrd_stdstringT filename, char* argv0);
	static jrd_stdstringT ExtractDirectoryFromFile(jrd_stdstringT filename);
	static jrd_stdstringT ExtractBaseNameFomFile(jrd_stdstringT filename);
	static bool CanWeWriteInDirectory(jrd_stdstringT dirpath);
	static jrd_stdstringT GetSafeDirectoryForWriting(jrd_stdstringT subdirpath);
	static jrd_stdstringT GetExeFullPathDirectory(char* argv0);
	static jrd_stdstringT GetCurrentWorkingDirectory();
	static bool SplitOffTopDir(jrd_stdstringT &path, jrd_stdstringT &topdir);
protected:
	// helpers
	void SetJrdType(const JrdType inmtype);
	void SetActivityModString(const char *inactivity);
	// formatting and output
	void DoVa(va_list &va, const char *formatx);
	void EscapeCharp(char *str,int maxlen);
	void DoFormattedOutput(const char *userstring);
public:
	// filters
	static void SetFilter_OnlyErrors(bool val) {filter_onlyerrors=val;};
	static void SetFilter_ShowAllBlocks(bool val) {filter_showallblocks=val;};
	static bool get_Filter_OnlyErrors() {return filter_onlyerrors;};
	static bool get_Filter_ShowAllBlocks() {return filter_showallblocks;};
protected:
	void UpdateErrorCounts();
public:
	// invoke the formatter with printf style argument line
	void operator()(const jrd_stdstringT str);
	void operator()(const char* format, ... );
	// alternative argument line with special parameters OutputDebugPrintf(MessageType,SeverityNumber,...)
	void operator()(const JrdType inmtype , const jrd_stdstringT str);
	void operator()(const JrdType inmtype , const char* format, ... );
	void operator()(const JrdType inmtype , const int inseverity, const jrd_stdstringT str);
	void operator()(const JrdType inmtype , const int inseverity, const char* format, ... );
	// alternative argument line with special parameters OutputDebugPrintf(MessageType,SubTypeString,ActivityString,SeverityNumber,...)
	void operator()(const JrdType inmtype , const char *dmsubtypestr, const char *inactivity, const int inseverity, const jrd_stdstringT str);
	void operator()(const JrdType inmtype , const char *dmsubtypestr, const char *inactivity, const int inseverity, const char* format, ... );
	// asserts
	void operator()(const JrEnumDummyFunctionDisambiguator_Assert dummyvar , const bool assertnotverify, const bool assertcondition, const char *assertstring);
	// test support
	void operator()(const JrdTestMsgType inmtype, const bool assertcondition, const char *assertstring, const char *assertstring2);
public:
	// block report helper
	static bool DebugBlockPushPointer(class JrDebugBlockObjectData *datum);
	static bool DebugBlockPopPointer(class JrDebugBlockObjectData *datum);
	static class JrDebugBlockObjectData* DebugBlockPopPointer(const char *labelstr,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3);
	static void DebugBlockReport(const char *labelstr);
	static void PopAllDebugBlocks();
public:
	// unit testing
	static void RegisterTestFunction(JrDebugRegisteredTestObj* rtop);
	static void RunTests(const char *inkeywordstring="all");
	static void ListTests();
	static bool RanTests();
	static void RunTestInternal(JrDebugRegisteredTestObj* rtop, bool catchexception=true);
	static void RunTestByPointer(JrdFuncpVoidRetT fp);
public:
	// output formatter helpers
	static void RegisterFormatterFactory(JrDebugRegisteredOutputFormatter* globalformatterp);
	static void AddInstantiatedFormatter(JrDebugRegisteredOutputFormatter* newp);
	static void ListFormatters();
	static void ShutdownFormatters();
	static JrDebugRegisteredOutputFormatter* InstantiateFormatter(const char* formattertypename,const char *optionstring="",const char *infilename="",bool stdoutflag=true,bool systemoutflag=false);
public:
	// snprintf replacements for when compiler doesnt support snprintf
	static int mysnprintf_rep(char *outstr, int maxlen, const char* format, ...);
	static int myvsnprintf_rep(char *outstr, int maxlen, const char* format, va_list &va );
public:
	// win32 console redirection
	#ifdef _WIN32JRD
		static void Win32RedirectConsoleIfNesc();
		static long JrdHandleToLong(HANDLE hnd);
	#endif
};
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// single global instance (needed so that we can get a destructor called on exit to close any open file)
extern JrDebug *GlobalJrDebugp;
//---------------------------------------------------------------------------








//---------------------------------------------------------------------------
// streaming debug object
typedef std::ostringstream jrd_stdostringstreamT;
class JrDebugs : public jrd_stdostringstreamT
{
protected:
	// pointer to a constructed JrDebug object which will do the work for us
	JrDebug *jrd;
	// we use state to hold optional info to send to JrDebug class when it comes time to output
	JrdType mtype;
	jrd_stdstringT dmsubtypestr;
	jrd_stdstringT activity;
	int severity;
	bool extrainfo;
public:
	// constructor and destructor, just forwards to the JrDebug_stringbuf
	JrDebugs(const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname = "", const char *infunctionname2="", const char *infunctionname3="", JrdType inmtype=JrdMessage);
	JrDebugs();
	~JrDebugs();
public:
	void setvalues(const JrdType inmtype , const char *indmsubtypestr, const char *inactivity, const int inseverity);
public:
	// sneaky use of operator() to set some additional parameters
	JrDebugs& operator()();
	JrDebugs& operator()(const JrdType inmtype , const int inseverity=JrDebugDefaultLevel);
	JrDebugs& operator()(const JrdType inmtype , const char *indmsubtypestr, const char *inactivity="", const int inseverity=JrDebugDefaultLevel);
	//
	JrDebugs& operator()(bool testcondition);
	JrDebugs& operator()(bool testcondition, const JrdType inmtype , const int inseverity=JrDebugDefaultLevel);
	JrDebugs& operator()(bool testcondition, const JrdType inmtype , const char *indmsubtypestr, const char *inactivity="", const int inseverity=JrDebugDefaultLevel);
};
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// DebugBlocks work by creating an object on the stack which announces the
//  start of the block; when the object goes out of scope at the end of the
//  block, it triggers the reporting of the end timing.
// We actually use a lightweight wrapper class that can be very easily
//  constructed on the stack, so that when debugging is compiled in but
//  disabled, it still has the minimal impact on runtime performance.

// this is the class that does the real work
class JrDebugBlockObjectData
{
private:
	// to keep track of starting time
	clock_t startclock;
	// info
	jrd_stdstringT filenamestring;
	jrd_stdstringT functionnamestring;
	jrd_stdstringT timestampstring;
	jrd_stdstringT label;
	int linenumber;
	//  is this a pushed blockobject or a manually set one (this affects how we handle setactivity() calls)
	bool isScopedelete;
	// does it need a closing announce?
	bool announceatend;	
	// parent
	JrDebugBlockObjectData *parentp;
	// for reporting
	int childerrors_block;
	int childerrors_immediate;
	int childerrors_all;
	int childs;
public:
	// constructor and destructor
	JrDebugBlockObjectData(const jrd_stdstringT str,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3, bool inisScopedelete);
	~JrDebugBlockObjectData();
public:
	// error maintenance
	void AddErrors(int inchilderrors_immediate, int inchilderrors_all, int inchilderrors_block);
	void IncError() {++childerrors_immediate;++childerrors_all;};
public:
	// reporting
	void report(const char* labelstr);
	void report_start();	
	void report_end();
	void report_error(const char *str);
public:
	// accessors
	void set_parentp(class JrDebugBlockObjectData* inparentp) {parentp=inparentp;};
	class JrDebugBlockObjectData* get_parentp() {return parentp;};
	bool get_isScopedelete() {return isScopedelete;};
	bool get_announceatend() {return announceatend;};
	bool doesMatchLabel(const char* labelstr);
	jrd_stdstringT get_label() {return label;};
	const char* get_labelcstr() {return label.c_str();};
	int get_childs() {return childs;};
	int get_childerrors_all() {return childerrors_all;};
	int get_childerrors_block() {return childerrors_block;};
};


// this is the lightweight wrapper; if debugging is disabled the init() will never be called, and object will just be constructed and destructed
//  wheras if debugging is enabled, init() will be called, etc.
class JrDebugBlockObject
{
private:
	// pointer to data
	JrDebugBlockObjectData *datum;
public:
	// constructor and destructor
	JrDebugBlockObject() : datum(NULL) {;};
	~JrDebugBlockObject() {if (datum) delete datum;};
public:
	// announce whenever you want
	void init(const jrd_stdstringT &str,const char* infilename, int ilinenum , const char *intimestamp , const char *infunctionname, const char *infunctionname2, const char *infunctionname3 ,bool inisScopedelete) {datum = new JrDebugBlockObjectData(str,infilename,ilinenum,intimestamp,infunctionname,infunctionname2,infunctionname3,inisScopedelete);};
};
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// Helper callback object - can be used if user code want to intercept events
//  To use, derive your own version of this class, create an instance of it, and pass it to the static
//  function JrDebug::SetJrDebugCallbackPointer(class JrDebugCallback *p);
// Callback should return true to process as normal, or false to not send output as normal
class JrDebugCallback {
public:
	virtual bool EventNotify(const char *fulltext)=0;
};
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// this is supposed to insure that we have 1 and only 1 global jrdebug global instance
//  and make sure it is created before we refer to it.
class JrDebugSingletonInsurer {
	bool globalkill;
public:
	// only 1 instance of singletoninsurer should ever be called with true (a global instance)
	JrDebugSingletonInsurer(bool isglobalkill=true) {if (GlobalJrDebugp==NULL) {GlobalJrDebugp = new JrDebug(true);}; globalkill=isglobalkill;};
	// we only clean up the global debug on the deletion of the last true globally instantiate insurer
	~JrDebugSingletonInsurer() {if (GlobalJrDebugp!=NULL && globalkill==true) { delete GlobalJrDebugp; GlobalJrDebugp=NULL; globalkill=false;} };
};
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// helper class for autoregistering tests - note we need to make sure that
//  the global jrdebug is created before we execute this
class JrDebugRegisteredTestObj {
protected:
	jrd_stdstringT simplename;
	jrd_stdstringT desc,keywordstring,fname;
	jrd_stdstringT filename,timestamp;
	int linenum;
	JrDebugRegisteredTestObj* nextrtop;
	// function pointer
	JrdFuncpVoidRetT fpoint;
public:
	// constructor (for normal scoped construction, used for temp creation of test objects)
	JrDebugRegisteredTestObj(const jrd_stdstringT insimplename,const jrd_stdstringT indesc,const jrd_stdstringT inkeywordstring,const jrd_stdstringT infname,JrdFuncpVoidRetT infpoint,const char* infilename, int ilinenum , const char *intimestamp);
	// constructor which allocates a new copy and gives it to global debug handler for future management
	JrDebugRegisteredTestObj(int dummyval_shouldrealloc,const jrd_stdstringT insimplename,const jrd_stdstringT indesc,const jrd_stdstringT inkeywordstring,const jrd_stdstringT infname,JrdFuncpVoidRetT infpoint,const char* infilename, int ilinenum , const char *intimestamp);
public:
	// accessors
	jrd_stdstringT getSimpleName() {return simplename;};
	void setNextp(JrDebugRegisteredTestObj* in_nextrtop) {nextrtop=in_nextrtop;};
	JrDebugRegisteredTestObj* getNextp() {return nextrtop;};
	const char* getDescriptioncharp() {return desc.c_str();};
	jrd_stdstringT getKeywords() {return keywordstring;};
	jrd_stdstringT getFname() {return fname;};
	jrd_stdstringT getFilename() {return filename;};
	jrd_stdstringT getDescription() {return desc;};
	int getLinenum() {return linenum;};
	JrdFuncpVoidRetT get_fpoint() {return fpoint;};
public:
	// invoke the function pointer
	void invoke() {fpoint();};
	// keyword match? (ATTN: unfinished = doesnt care about wholewords)
	bool doeskeymatch(const jrd_stdstringT matchstring);
public:
	// announce exception
	void AnnounceException(const char *typestr);
	void AnnounceException(const char *typestr,const char *whatstr);
};
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
#include <exception>
// #include <stdexcept>
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// unit test helpers
class JrdTestException : public std::exception
{
protected:
	jrd_stdstringT exceptionstring;
public:
	JrdTestException(char *inexceptionstring) throw();
	~JrdTestException() throw();
public:
	jrd_stdstringT getExceptionString() throw();
};
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// Output can be generated in different styles by registering output handlers

// We have different derived JrDebugRegisteredOutputFormatter objects which define how to format output
//  these are also instantiated and used to do actual output generator.
// Multiple output handlers can be instantiated and attached, and each one will fire in turn.
// Each one can be set to send its output to any or all of file,console,windows debug trace
class JrDebugRegisteredOutputFormatter {
protected:
	jrd_stdstringT filename;
	FILE* filepointer;
	bool enabled_systemout;
	bool enabled_fileout;
	bool enabled_stdout;
	bool appendmode;
protected:
	// keep track of # lines written
	int linesout;
protected:
	// linked list pointer
	JrDebugRegisteredOutputFormatter* nextp;
public:
	// constructor and destructor
	JrDebugRegisteredOutputFormatter();
	JrDebugRegisteredOutputFormatter(bool isfactory);
	virtual ~JrDebugRegisteredOutputFormatter();
public:
	// initialization
	bool Initialize(const char *optionstring,const char *infilename,bool stdoutflag,bool systemoutflag);
public:
	// accessors
	JrDebugRegisteredOutputFormatter* get_nextp() {return nextp;};
	void set_nextp(JrDebugRegisteredOutputFormatter* val) {nextp=val;};
	bool isStdoutEnabled() {return enabled_stdout;};
public:
	jrd_stdstringT GetHeaderAnnounceString();
public:
	// naming
	bool doesnamematch(const char* formattertypename);
public:
	// file creation helper
	bool CreateOutputFile(const char *fname,bool appendmode);
	// Send line to file/stdout/sytem debug message
	void DoOutput(const char *str);
	void DoOutput(const char *str,int repetitions);
public:
	// file name helper
	static jrd_stdstringT ReplaceFileNameArgs(jrd_stdstringT &infname);
public:
	// Functions that should be implemented by derived classes
public:
	// factory function, create a copy of derived type
	virtual JrDebugRegisteredOutputFormatter* MakeNew() {return new JrDebugRegisteredOutputFormatter;};
	// return name of style for use with commandline instantiation
	virtual const char* GetStyleName() {return "_OutputFormatterBaseStyle";};
	virtual const char* GetStyleDescription() {return "Base class - you shouldn't instantiate this.";};
	virtual const char* GetStyleOptionsDescription() {return "";};
	virtual const char* GetDefaultFileName() {return "jrdoutput.txt";};
	virtual const char* GetDefaultFileExt() {return "txt";};
public:
	// Shutdown and startup
	virtual bool Startup(const char *optionstring);
	virtual bool Shutdown();
public:
	// HandleDebugMessage should format the user's string and call DoOutput one or more times
	// Note it will be invoked with any user % args already repalced
	// This is the main function that derived classes will override
	virtual void HandleDebugMessage(JrDebug *jrd,const char *str);
};
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// Simple human-readable plaintext output
class JrDebugRegisteredOutputFormatter_PlainText : public JrDebugRegisteredOutputFormatter
{
public:
	// constructor and destructor
	JrDebugRegisteredOutputFormatter_PlainText(bool isfactory);
	virtual ~JrDebugRegisteredOutputFormatter_PlainText() {;};
protected:
	bool flag_doshowtypeprefix,flag_doshowtimestamps;
public:
	// factory function, create a copy of derived type
	virtual JrDebugRegisteredOutputFormatter* MakeNew() {return new JrDebugRegisteredOutputFormatter_PlainText(false);};
	// return name of style for use with commandline instantiation
	virtual const char* GetStyleName() {return "text";};
	virtual const char* GetStyleDescription() {return "simple plain text output.";};
	virtual const char* GetStyleOptionsDescription() {return "";};
	virtual const char* GetDefaultFileName() {return "jrdoutput.txt";};
	virtual const char* GetDefaultFileExt() {return "txt";};
public:
	// Shutdown and startup
	virtual bool Startup(const char *optionstring);
	virtual bool Shutdown();
public:
	// HandleDebugMessage should format the user's string and call DoOutput one or more times
	// Note it will be invoked with any user % args already repalced
	// This is the main function that derived classes will override
	virtual void HandleDebugMessage(JrDebug *jrd,const char *str);
};



// Internal jrdebug style, used for easy parsing by gui tool, similar in approach to
//  xml but simpler, more compact, and unlikely to conflict with normal text
//  (basically it just uses named columns/attributes).
class JrDebugRegisteredOutputFormatter_Jrd : public JrDebugRegisteredOutputFormatter
{
public:
	// constructor and destructor
	JrDebugRegisteredOutputFormatter_Jrd(bool isfactory);
	virtual ~JrDebugRegisteredOutputFormatter_Jrd() {;};
public:
	// factory function, create a copy of derived type
	virtual JrDebugRegisteredOutputFormatter* MakeNew() {return new JrDebugRegisteredOutputFormatter_Jrd(false);};
	// return name of style for use with commandline instantiation
	virtual const char* GetStyleName() {return "jrd";};
	virtual const char* GetStyleDescription() {return "jrdebug style output, for use with the gui tool";};
	virtual const char* GetStyleOptionsDescription() {return "";};
	virtual const char* GetDefaultFileName() {return "jrdoutput.log";};
	virtual const char* GetDefaultFileExt() {return "log";};
public:
	// Shutdown and startup
	virtual bool Startup(const char *optionstring);
	virtual bool Shutdown();
public:
	// HandleDebugMessage should format the user's string and call DoOutput one or more times
	// Note it will be invoked with any user % args already repalced
	// This is the main function that derived classes will override
	virtual void HandleDebugMessage(JrDebug *jrd,const char *str);
};



// Generate output that emulates compiler error formats, usedful for reading from the ide
class JrDebugRegisteredOutputFormatter_Compiler : public JrDebugRegisteredOutputFormatter
{
private:
	// emulation mode for output
	enum JrdEmulationModeType { JrDebug_CompilerOutput_EmulationMode_Generic, JrDebug_CompilerOutput_EmulationMode_Gcc, JrDebug_CompilerOutput_EmulationMode_Bcc, JrDebug_CompilerOutput_EmulationMode_Msvc };
	JrdEmulationModeType emulationmode;
public:
	// constructor and destructor
	JrDebugRegisteredOutputFormatter_Compiler(bool isfactory);
	virtual ~JrDebugRegisteredOutputFormatter_Compiler() {;};
public:
	// factory function, create a copy of derived type
	virtual JrDebugRegisteredOutputFormatter* MakeNew() {return new JrDebugRegisteredOutputFormatter_Compiler(false);};
	// return name of style for use with commandline instantiation
	virtual const char* GetStyleName() {return "comp";};
	virtual const char* GetStyleDescription() {return "compiler style output, for use with a c++ ide";};
	virtual const char* GetStyleOptionsDescription() {return "options: gcc|bcc|msvc|generic";};
	virtual const char* GetDefaultFileName() {return "jrdoutput.log";};
	virtual const char* GetDefaultFileExt() {return "log";};
public:
	// Shutdown and startup
	virtual bool Startup(const char *optionstring);
	virtual bool Shutdown();
public:
	// HandleDebugMessage should format the user's string and call DoOutput one or more times
	// Note it will be invoked with any user % args already repalced
	// This is the main function that derived classes will override
	virtual void HandleDebugMessage(JrDebug *jrd,const char *str);
};



// Generate output in csv format
class JrDebugRegisteredOutputFormatter_Csv : public JrDebugRegisteredOutputFormatter
{
private:
	// emulation mode for output
	enum JrdEmulationModeType { JrDebug_CompilerOutput_EmulationMode_Generic, JrDebug_CompilerOutput_EmulationMode_Gcc, JrDebug_CompilerOutput_EmulationMode_Bcc, JrDebug_CompilerOutput_EmulationMode_Msvc };
	JrdEmulationModeType emulationmode;
public:
	// constructor and destructor
	JrDebugRegisteredOutputFormatter_Csv(bool isfactory);
	virtual ~JrDebugRegisteredOutputFormatter_Csv() {;};
public:
	// factory function, create a copy of derived type
	virtual JrDebugRegisteredOutputFormatter* MakeNew() {return new JrDebugRegisteredOutputFormatter_Csv(false);};
	// return name of style for use with commandline instantiation
	virtual const char* GetStyleName() {return "csv";};
	virtual const char* GetStyleDescription() {return "comma separated fields output, for use with generic spreadsheet/database";};
	virtual const char* GetStyleOptionsDescription() {return "";};
	virtual const char* GetDefaultFileName() {return "jrdoutput.csv";};
	virtual const char* GetDefaultFileExt() {return "csv";};
public:
	// Shutdown and startup
	virtual bool Startup(const char *optionstring);
	virtual bool Shutdown();
public:
	// HandleDebugMessage should format the user's string and call DoOutput one or more times
	// Note it will be invoked with any user % args already repalced
	// This is the main function that derived classes will override
	virtual void HandleDebugMessage(JrDebug *jrd,const char *str);
};



// Nice pretty Html output
class JrDebugRegisteredOutputFormatter_Html : public JrDebugRegisteredOutputFormatter
{
private:
	// keeping track of output
	bool lastblank;
    bool wantblank;
	bool wantstartblank;
public:	
	// constructor and destructor
	JrDebugRegisteredOutputFormatter_Html(bool isfactory);
	virtual ~JrDebugRegisteredOutputFormatter_Html() {;};
public:
	// factory function, create a copy of derived type
	virtual JrDebugRegisteredOutputFormatter* MakeNew() {return new JrDebugRegisteredOutputFormatter_Html(false);};
	// return name of style for use with commandline instantiation
	virtual const char* GetStyleName() {return "html";};
	virtual const char* GetStyleDescription() {return "table-based html";};
	virtual const char* GetStyleOptionsDescription() {return "";};
	virtual const char* GetDefaultFileName() {return "jrdoutput.html";};
	virtual const char* GetDefaultFileExt() {return "html";};
public:
	// Shutdown and startup
	virtual bool Startup(const char *optionstring);
	virtual bool Shutdown();
public:
	// HandleDebugMessage should format the user's string and call DoOutput one or more times
	// Note it will be invoked with any user % args already repalced
	// This is the main function that derived classes will override
	virtual void HandleDebugMessage(JrDebug *jrd,const char *str);
};



// Xml formatted output
class JrDebugRegisteredOutputFormatter_Xml : public JrDebugRegisteredOutputFormatter
{
public:
	// constructor and destructor
	JrDebugRegisteredOutputFormatter_Xml(bool isfactory);
	virtual ~JrDebugRegisteredOutputFormatter_Xml() {;};
public:
	// factory function, create a copy of derived type
	virtual JrDebugRegisteredOutputFormatter* MakeNew() {return new JrDebugRegisteredOutputFormatter_Xml(false);};
	// return name of style for use with commandline instantiation
	virtual const char* GetStyleName() {return "xml";};
	virtual const char* GetStyleDescription() {return "xml formatted output";};
	virtual const char* GetStyleOptionsDescription() {return "";};
	virtual const char* GetDefaultFileName() {return "jrdoutput.xml";};
	virtual const char* GetDefaultFileExt() {return "xml";};
public:
	// Shutdown and startup
	virtual bool Startup(const char *optionstring);
	virtual bool Shutdown();
public:
	// HandleDebugMessage should format the user's string and call DoOutput one or more times
	// Note it will be invoked with any user % args already repalced
	// This is the main function that derived classes will override
	virtual void HandleDebugMessage(JrDebug *jrd,const char *str);
};
//---------------------------------------------------------------------------
